@@ -29,7 +29,7 @@ import pathlib
 import xml.etree.ElementTree as ET
 from collections import defaultdict
 from enum import Enum
-from typing import Any, ClassVar, Dict, Iterable, List, Optional, Tuple, Type, TypedDict, TypeVar, Union
+from typing import Any, ClassVar, Dict, Iterable, Type, TypedDict, TypeVar, Union
 
 import numpy as np
 import shapely
@@ -57,9 +57,9 @@ class GeoJsonDict(TypedDict):
     TypedDict for standard GeoJSON output
     """
 
-    id: Optional[str]
+    id: str | None
     type: str
-    features: List[Dict[str, Union[str, Dict[str, str]]]]
+    features: list[Dict[str, Union[str, Dict[str, str]]]]
 
 
 class Point(shapely.geometry.Point):
@@ -70,14 +70,14 @@ class Point(shapely.geometry.Point):
     )  # slots must be the same for assigning __class__ - https://stackoverflow.com/a/52140968
     name: str  # For documentation generation and static type checking
 
-    def __init__(self, coord: Union[shapely.geometry.Point, Tuple[float, float]], label: Optional[str] = None) -> None:
+    def __init__(self, coord: Union[shapely.geometry.Point, tuple[float, float]], label: str | None = None) -> None:
         self._id_to_attrs[str(id(self))] = dict(label=label)
 
     @property
     def type(self):
         return AnnotationType.POINT
 
-    def __new__(cls, coord: Tuple[float, float], *args, **kwargs) -> "Point":
+    def __new__(cls, coord: tuple[float, float], *args, **kwargs) -> "Point":
         point = super().__new__(cls, coord)
         point.__class__ = cls
         return point
@@ -103,16 +103,14 @@ class Polygon(shapely.geometry.Polygon):
     )  # slots must be the same for assigning __class__ - https://stackoverflow.com/a/52140968
     name: str  # For documentation generation and static type checking
 
-    def __init__(
-        self, coord: Union[shapely.geometry.Polygon, Tuple[float, float]], label: Optional[str] = None
-    ) -> None:
+    def __init__(self, coord: Union[shapely.geometry.Polygon, tuple[float, float]], label: str | None = None) -> None:
         self._id_to_attrs[str(id(self))] = dict(label=label)
 
     @property
     def type(self):
         return AnnotationType.POLYGON
 
-    def __new__(cls, coord: Tuple[float, float], *args, **kwargs) -> "Point":
+    def __new__(cls, coord: tuple[float, float], *args, **kwargs) -> "Point":
         point = super().__new__(cls, coord)
         point.__class__ = cls
         return point
@@ -130,14 +128,16 @@ class Polygon(shapely.geometry.Polygon):
         return f"{self.label}, {self.wkt}"
 
 
-def shape(coordinates, label):
+def shape(coordinates, label, multiplier: float = 1.0):
     geom_type = coordinates.get("type").lower()
     if geom_type == "point":
-        return [Point(coordinates["coordinates"], label=label)]
+        return [Point(np.asarray(coordinates["coordinates"]) * multiplier, label=label)]
     elif geom_type == "polygon":
-        return [Polygon(coordinates["coordinates"][0], label=label)]
+        return [Polygon(np.asarray(coordinates["coordinates"][0]) * multiplier, label=label)]
     elif geom_type == "multipolygon":
-        multi_polygon = shapely.geometry.MultiPolygon([[c[0], c[1:]] for c in coordinates["coordinates"]])
+        multi_polygon = shapely.geometry.MultiPolygon(
+            [[np.asarray(c[0]) * multiplier, np.asarray(c[1:]) * multiplier] for c in coordinates["coordinates"]]
+        )
         return [Polygon(_, label=label) for _ in multi_polygon.geoms]
     else:
         raise NotImplementedError(f"Not support geom_type {geom_type}")
@@ -190,10 +190,10 @@ class WsiSingleLabelAnnotation:
     def as_strtree(self) -> STRtree:
         return STRtree(self._annotations)
 
-    def as_list(self) -> List:
+    def as_list(self) -> list:
         return self._annotations
 
-    def as_json(self) -> List[Dict[str, Any]]:
+    def as_json(self) -> list[Dict[str, Any]]:
         """
         Return the annotation as json format.
 
@@ -238,7 +238,7 @@ class WsiSingleLabelAnnotation:
 class WsiAnnotations:
     """Class to hold the annotations of all labels specific label for a whole slide image."""
 
-    def __init__(self, annotations: List[WsiSingleLabelAnnotation]):
+    def __init__(self, annotations: list[WsiSingleLabelAnnotation]):
         self.available_labels = sorted([annotation.label for annotation in annotations])
         if len(set(self.available_labels)) != len(self.available_labels):
             raise ValueError(
@@ -251,7 +251,7 @@ class WsiAnnotations:
         # Now we have a dict of label: annotations.
         self._annotation_trees = {label: self[label].as_strtree() for label in self.available_labels}
 
-    def filter(self, labels: Union[str, Union[List[str], Tuple[str]]]) -> None:
+    def filter(self, labels: Union[str, Union[list[str], tuple[str]]]) -> None:
         """
         Filter annotations based on the given label list.
 
@@ -269,9 +269,9 @@ class WsiAnnotations:
         self._annotations = {k: v for k, v in self._annotations.items() if k in _labels}
         self._annotation_trees = {k: v for k, v in self._annotation_trees.items() if k in _labels}
 
-    def relabel(self, labels: Tuple[Tuple[str, str], ...]) -> None:
+    def relabel(self, labels: tuple[tuple[str, str], ...]) -> None:
         """
-        Rename labels in the class. Preferably use the remapping during initialization of this class.
+        Rename labels in the class in-place.
 
         Parameters
         ----------
@@ -299,6 +299,27 @@ class WsiAnnotations:
         self._annotations = _annotations
         self._annotation_trees = {label: self[label].as_strtree() for label in self.available_labels}
 
+    @property
+    def bounding_box(self) -> tuple[tuple[float, float], tuple[float, float]]:
+        """
+        Return the bounding box of all annotations.
+
+        Returns
+        -------
+        tuple[tuple[float, float], tuple[float, float]]
+            Bounding box of the form ((x, y), (w, h)).
+        """
+        all_boxes = []
+        for label in self.available_labels:
+            curr_bboxes = self[label].bounding_boxes
+            for box_start, box_size in curr_bboxes:
+                max_x, max_y = box_start[0] + box_size[0], box_start[1] + box_size[1]
+                all_boxes.append(shapely.geometry.box(*box_start, max_x, max_y))
+
+        boxes_as_multipolygon = shapely.geometry.MultiPolygon(all_boxes)
+        min_x, min_y, max_x, max_y = boxes_as_multipolygon.bounds
+        return (min_x, min_y), (max_x - min_x, max_y - min_y)
+
     def copy(self) -> WsiAnnotations:
         """Make a copy of the object."""
         return copy.deepcopy(self)
@@ -308,7 +329,6 @@ class WsiAnnotations:
         cls: Type[_TWsiAnnotations],
         geojsons: Union[PathLike, Iterable[PathLike]],
         scaling: float | None = None,
-        remap_labels: Dict[str, str] | None = None,
     ) -> _TWsiAnnotations:
         """
         Constructs an WsiAnnotations object from geojson.
@@ -320,9 +340,6 @@ class WsiAnnotations:
             object.
         scaling : float, optional
             The scaling to apply to the annotations.
-        remap_labels : dict, optional
-            Dictionary mapping read labels to another label. Multiple labels can map to the same value. If the
-            label is not in the dictionary this label will be ignored.
 
         Returns
         -------
@@ -330,9 +347,11 @@ class WsiAnnotations:
 
         """
         data = defaultdict(list)
-        _remap_labels = {} if not remap_labels else remap_labels
         _scaling = 1.0 if not scaling else scaling
-        _geojsons = [geojsons] if not isinstance(geojsons, (tuple, list, Iterable)) else geojsons
+        if isinstance(geojsons, str):
+            _geojsons: Iterable[Any] = [pathlib.Path(geojsons)]
+
+        _geojsons = [geojsons] if not isinstance(geojsons, (tuple, list)) else geojsons
         for idx, path in enumerate(_geojsons):
             path = pathlib.Path(path)
             if not path.exists():
@@ -341,18 +360,13 @@ class WsiAnnotations:
             with open(path, "r", encoding="utf-8") as annotation_file:
                 geojson_dict = json.load(annotation_file)["features"]
                 for x in geojson_dict:
-                    coordinates = np.asarray(x["geometry"]["coordinates"]) * _scaling
-                    x["geometry"]["coordinates"] = coordinates.tolist()
                     _label = x["properties"]["classification"]["name"]
-
-                    if remap_labels and _label in _remap_labels:
-                        _label = _remap_labels[_label]
-                    _geometry = shape(x["geometry"], label=_label)
+                    _geometry = shape(x["geometry"], label=_label, multiplier=_scaling)
                     for _ in _geometry:
                         data[_label].append(_)
 
-        # It is assume that a specific label can only be one type (point or polygon)
-        annotations: List[WsiSingleLabelAnnotation] = [
+        # It is assumed that a specific label can only be one type (point or polygon)
+        annotations: list[WsiSingleLabelAnnotation] = [
             WsiSingleLabelAnnotation(label=k, type=data[k][0].type, coordinates=data[k]) for k in data.keys()
         ]
 
@@ -363,7 +377,6 @@ class WsiAnnotations:
         cls,
         asap_xml: PathLike,
         scaling: float | None = None,
-        remap_labels: Dict[str, str] | None = None,
     ):
         """
         Read annotations as an ASAP XML file.
@@ -372,14 +385,11 @@ class WsiAnnotations:
         Parameters
         ----------
         asap_xml
-        scaling
-        remap_labels : dict, optional
-            Dictionary mapping read labels to another label. Multiple labels can map to the same value. If the
-            label is not in the dictionary this label will be ignored.
+        scaling : float, optional
 
         Returns
         -------
-
+        WsiAnnotations
         """
         _ASAP_TYPES = {
             "polygon": AnnotationType.POLYGON,
@@ -388,7 +398,6 @@ class WsiAnnotations:
             "spline": AnnotationType.POLYGON,
             "pointset": AnnotationType.POINT,
         }
-        _remap_labels = {} if not remap_labels else remap_labels
 
         tree = ET.parse(asap_xml)
         opened_annotation = tree.getroot()
@@ -400,12 +409,6 @@ class WsiAnnotations:
                     continue
                 label = child.attrib.get("PartOfGroup").lower().strip()  # type: ignore
 
-                # If we have a label map and there is nothing defined, then continue.
-                if _remap_labels:
-                    if label not in _remap_labels:
-                        continue
-                    label = _remap_labels[label]
-
                 _type = child.attrib.get("Type").lower()  # type: ignore
                 annotation_type = _ASAP_TYPES[_type]
                 coordinates = _parse_asap_coordinates(child, annotation_type, scaling=scaling)
@@ -455,7 +458,7 @@ class WsiAnnotations:
     def __getitem__(self, label: str) -> WsiSingleLabelAnnotation:
         return self._annotations[label]
 
-    def as_geojson(self, split_per_label=False) -> Union[GeoJsonDict, List[Tuple[str, GeoJsonDict]]]:
+    def as_geojson(self, split_per_label=False) -> Union[GeoJsonDict, list[tuple[str, GeoJsonDict]]]:
         """
         Output the annotations as proper geojson.
 
@@ -509,10 +512,10 @@ class WsiAnnotations:
 
     def read_region(
         self,
-        coordinates: Union[np.ndarray, Tuple[GenericNumber, GenericNumber]],
+        coordinates: Union[np.ndarray, tuple[GenericNumber, GenericNumber]],
         scaling: float,
-        region_size: Union[np.ndarray, Tuple[GenericNumber, GenericNumber]],
-    ) -> List[Union[Polygon, Point]]:
+        region_size: Union[np.ndarray, tuple[GenericNumber, GenericNumber]],
+    ) -> list[Union[Polygon, Point]]:
         """Reads the region of the annotations. API is the same as `dlup.SlideImage` so they can be used in conjunction.
 
         The process is as follows:
@@ -536,7 +539,7 @@ class WsiAnnotations:
 
         Returns
         -------
-        List[Tuple[str, ShapelyTypes]]
+        list[tuple[str, ShapelyTypes]]
             List of tuples denoting the name of the annotation and a shapely object.
 
         Examples
@@ -577,6 +580,7 @@ class WsiAnnotations:
         for annotation_name, annotation in filtered_annotations:
             if annotation.is_valid is False:
                 annotation = make_valid(annotation)
+
             crop_func = _POSTPROCESSORS[self[annotation_name].type]
             if crop_func is not None:
                 curr_area = annotation.area
@@ -593,7 +597,7 @@ class WsiAnnotations:
 
         transformation_matrix = [scaling, 0, 0, scaling, -coordinates[0], -coordinates[1]]
 
-        output: List[Union[Polygon, Point]] = []
+        output: list[Union[Polygon, Point]] = []
         for annotation_name, annotation in cropped_annotations:
             annotation = shapely.affinity.affine_transform(annotation, transformation_matrix)
             # It can occur that single polygon annotations result in being points after being intersected.
@@ -640,10 +644,11 @@ class WsiAnnotations:
     def __contains__(self, item):
         return item in self.available_labels
 
-    def __add__(self, other) -> WsiAnnotations:
+    def __add__(self, other: WsiAnnotations) -> WsiAnnotations:
         if set(self.available_labels).intersection(other.available_labels) != set():
             raise AnnotationError(
-                f"Can only add annotations with different labels. Use `.relabel` or relabel during construction of the object."
+                f"Can only add annotations with different labels. "
+                f"Use `.relabel` or relabel during construction of the object."
             )
 
         curr_annotations = list(self._annotations.values())
