@@ -11,7 +11,7 @@ import collections
 import functools
 import itertools
 import pathlib
-from typing import Callable, Dict, Generic, Iterable, List, Optional, Tuple, TypedDict, TypeVar, Union, cast
+from typing import Callable, Generic, Iterable, TypedDict, TypeVar, Union, cast
 
 import numpy as np
 import PIL
@@ -28,20 +28,20 @@ from dlup.tools import ConcatSequences, MapSequence
 T_co = TypeVar("T_co", covariant=True)
 T = TypeVar("T")
 _BaseAnnotationTypes = Union[SlideImage, WsiAnnotations]
-_AnnotationTypes = Union[List[Tuple[str, _BaseAnnotationTypes]], _BaseAnnotationTypes]
+_AnnotationTypes = Union[list[tuple[str, _BaseAnnotationTypes]], _BaseAnnotationTypes]
 _LabelTypes = Union[str, bool, int, float]
 
 
 class StandardTilingFromSlideDatasetSample(TypedDict):
     image: PIL.Image.Image
-    coordinates: Tuple[int, int]
+    coordinates: tuple[int, int]
     mpp: float
     path: pathlib.Path
     region_index: int
 
 
 class RegionFromSlideDatasetSample(StandardTilingFromSlideDatasetSample):
-    grid_local_coordinates: Tuple
+    grid_local_coordinates: tuple
     grid_index: int
 
 
@@ -93,9 +93,9 @@ class ConcatDataset(Dataset[T_co]):
 
     """
 
-    datasets: List[Dataset[T_co]]
-    cumulative_sizes: List[int]
-    wsi_indices: Dict[str, range]
+    datasets: list[Dataset[T_co]]
+    cumulative_sizes: list[int]
+    wsi_indices: dict[str, range]
 
     @staticmethod
     def cumsum(sequence):
@@ -116,25 +116,42 @@ class ConcatDataset(Dataset[T_co]):
                 raise ValueError("ConcatDataset requires datasets to be indexable.")
         self.cumulative_sizes = self.cumsum(self.datasets)
 
-    def __len__(self):
-        return self.cumulative_sizes[-1]
+    def index_to_dataset(self, idx: int) -> tuple[Dataset, int]:
+        """Returns the dataset and the index of the sample in the dataset.
 
-    def __getitem__(self, idx):
+        Parameters
+        ----------
+        idx : int
+            Index of the sample in the concatenated dataset.
+
+        Returns
+        -------
+        tuple[Dataset, int]
+            Dataset and index of the sample in the dataset.
+        """
         if idx < 0:
             if -idx > len(self):
                 raise ValueError("absolute value of index should not exceed dataset length")
             idx = len(self) + idx
         dataset_idx = bisect.bisect_right(self.cumulative_sizes, idx)
         sample_idx = idx if dataset_idx == 0 else idx - self.cumulative_sizes[dataset_idx - 1]
-        return self.datasets[dataset_idx][sample_idx]
+        return self.datasets[dataset_idx], sample_idx
+
+    def __len__(self):
+        return self.cumulative_sizes[-1]
+
+    def __getitem__(self, idx):
+        """Returns the sample at the given index."""
+        dataset, sample_idx = self.index_to_dataset(idx)
+        return dataset[sample_idx]
 
 
 LRU_CACHE_SIZE = 32
 
 
 @functools.lru_cache(LRU_CACHE_SIZE)
-def _get_cached_slide_image(path: pathlib.Path, backend):
-    return SlideImage.from_file_path(path, backend=backend)
+def _get_cached_slide_image(path: pathlib.Path, backend, **kwargs):
+    return SlideImage.from_file_path(path, backend=backend, **kwargs)
 
 
 class SlideImageDatasetBase(Dataset[T_co]):
@@ -153,12 +170,14 @@ class SlideImageDatasetBase(Dataset[T_co]):
         path: pathlib.Path,
         regions: collections.abc.Sequence,
         crop: bool = False,
-        mask: Optional[Union[SlideImage, np.ndarray, WsiAnnotations]] = None,
-        mask_threshold: float = 0.1,
-        annotations: Optional[Union[List[_AnnotationTypes], _AnnotationTypes]] = None,
-        labels: Optional[List[Tuple[str, _LabelTypes]]] = None,
-        transform: Optional[Callable] = None,
+        mask: Union[SlideImage, np.ndarray, WsiAnnotations] | None = None,
+        mask_threshold: float | None = 0.0,
+        output_tile_size: tuple[int, int] | None = None,
+        annotations: Union[list[_AnnotationTypes], _AnnotationTypes] | None = None,
+        labels: list[tuple[str, _LabelTypes]] | None = None,
+        transform: Callable | None = None,
         backend: Callable = ImageBackend.PYVIPS,
+        **kwargs,
     ):
         """
         Parameters
@@ -168,27 +187,37 @@ class SlideImageDatasetBase(Dataset[T_co]):
         regions :
             Sequence of rectangular regions as (x, y, h, w, mpp)
         crop :
-            Whether or not to crop overflowing tiles.
+            Whether to crop overflowing tiles.
         mask :
-            Binary mask used to filter each region toghether with a threshold.
-        mask_threshold :
-            0 every region is discarded, 1 requires the whole region to be foreground.
+            Binary mask used to filter each region together with a threshold.
+        mask_threshold : float, optional
+            Threshold to check against. The foreground percentage should be strictly larger than threshold.
+            If None anything is foreground. If 1, the region must be completely foreground.
+            Other values are in between, for instance if 0.5, the region must be at least 50% foreground.
+        output_tile_size: tuple[int, int], optional
+            If this value is set, this value will be used as the tile size of the output tiles. If this value
+            is different from the underlying grid, this tile will be extracted around the center of the region.
         annotations :
             Annotation classes.
         labels : list
             Image-level labels. Will be added to each individual tile.
         transform :
             Transforming function. To be used for augmentations or other model specific preprocessing.
+        **kwargs :
+            Keyword arguments get passed to the underlying slide image.
         """
         # We need to reuse the pah in order to re-open the image if necessary.
         self._path = path
         self._crop = crop
         self.regions = regions
 
+        self._output_tile_size = output_tile_size
+
         self.annotations = annotations
         self.labels = labels
         self.__transform = transform
         self._backend = backend
+        self._kwargs = kwargs
 
         # Maps from a masked index -> regions index.
         # For instance, let's say we have three regions
@@ -214,7 +243,7 @@ class SlideImageDatasetBase(Dataset[T_co]):
     @property
     def slide_image(self):
         """Return the cached slide image instance associated with this dataset."""
-        return _get_cached_slide_image(self.path, self._backend)
+        return _get_cached_slide_image(self.path, self._backend, **self._kwargs)
 
     def __getitem__(self, index):
         slide_image = self.slide_image
@@ -228,12 +257,20 @@ class SlideImageDatasetBase(Dataset[T_co]):
             region_index = index
 
         x, y, w, h, mpp = self.regions[region_index]
-        coordinates: Tuple[int, int] = x, y
-        region_size: Tuple[int, int] = w, h
+        coordinates: tuple[int | float, int | float] = x, y
+        region_size: tuple[int, int] = w, h
         scaling: float = slide_image.mpp / mpp
         region_view = slide_image.get_scaled_view(scaling)
         region_view.boundary_mode = BoundaryMode.crop if self.crop else BoundaryMode.zero
 
+        if self._output_tile_size is not None:
+            # If we have an output tile_size, we extract a region around the center of the given region.
+            output_tile_x, output_tile_y = self._output_tile_size
+            coordinates_x = x + w / 2 - output_tile_x / 2
+            coordinates_y = y + h / 2 - output_tile_y / 2
+            coordinates = (coordinates_x, coordinates_y)
+            region_size = self._output_tile_size
+
         region = region_view.read_region(coordinates, region_size)
 
         sample: StandardTilingFromSlideDatasetSample = {
@@ -277,7 +314,7 @@ def _coords_to_region(tile_size, target_mpp, key, coords):
 
 
 class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSample]):
-    """Example dataset dataset class that supports multiple ROIs.
+    """Example dataset class that supports multiple ROIs.
 
     This dataset can be used, for example, to tile your WSI on-the-fly using the `from_standard_tiling` function.
 
@@ -303,14 +340,16 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
     def __init__(
         self,
         path: pathlib.Path,
-        grids: List[Tuple[Grid, Tuple[int, int], float]],
+        grids: list[tuple[Grid, tuple[int, int], float]],
         crop: bool = False,
-        mask: Optional[Union[SlideImage, np.ndarray, WsiAnnotations]] = None,
-        mask_threshold: float = 0.1,
-        annotations: Optional[_AnnotationTypes] = None,
-        labels: Optional[List[Tuple[str, _LabelTypes]]] = None,
-        transform: Optional[Callable] = None,
+        mask: Union[SlideImage, np.ndarray, WsiAnnotations] | None = None,
+        mask_threshold: float | None = 0.0,
+        output_tile_size: tuple[int, int] | None = None,
+        annotations: _AnnotationTypes | None = None,
+        labels: list[tuple[str, _LabelTypes]] | None = None,
+        transform: Callable | None = None,
         backend: Callable = ImageBackend.PYVIPS,
+        **kwargs,
     ):
         self._grids = grids
         regions = []
@@ -327,8 +366,10 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
             mask_threshold=mask_threshold,
             annotations=annotations,
             labels=labels,
+            output_tile_size=output_tile_size,
             transform=None,
             backend=backend,
+            **kwargs,
         )
         self.__transform = transform
 
@@ -341,18 +382,20 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
         cls,
         path: pathlib.Path,
         mpp: float | None,
-        tile_size: Tuple[int, int],
-        tile_overlap: Tuple[int, int],
+        tile_size: tuple[int, int],
+        tile_overlap: tuple[int, int],
         tile_mode: TilingMode = TilingMode.overflow,
         grid_order: GridOrder = GridOrder.C,
         crop: bool = False,
-        mask: Optional[Union[SlideImage, np.ndarray, WsiAnnotations]] = None,
-        mask_threshold: float = 0.1,
-        rois: Optional[Tuple[Tuple[int, ...]]] = None,
-        annotations: Optional[_AnnotationTypes] = None,
-        labels: Optional[List[Tuple[str, _LabelTypes]]] = None,
-        transform: Optional[Callable] = None,
+        mask: Union[SlideImage, np.ndarray, WsiAnnotations] | None = None,
+        mask_threshold: float | None = 0.0,
+        output_tile_size: tuple[int, int] | None = None,
+        rois: tuple[tuple[int, ...]] | None = None,
+        annotations: _AnnotationTypes | None = None,
+        labels: list[tuple[str, _LabelTypes]] | None = None,
+        transform: Callable | None = None,
         backend: Callable = ImageBackend.PYVIPS,
+        **kwargs,
     ):
         """Function to be used to tile a WSI on-the-fly.
         Parameters
@@ -369,12 +412,17 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
             "skip" or "overflow". see `dlup.tiling.TilingMode` for more information
         grid_order : GridOrder
             Run through the grid either in C order or Fortran order.
-        crop :
-            Whether or not to crop overflowing tiles.
+        crop : bool
+             If overflowing tiles should be cropped.
         mask :
-            Binary mask used to filter each region toghether with a threshold.
-        mask_threshold :
-            0 every region is discarded, 1 requires the whole region to be foreground.
+            Binary mask used to filter each region together with a threshold.
+        mask_threshold : float, optional
+            Threshold to check against. The foreground percentage should be strictly larger than threshold.
+            If None anything is foreground. If 1, the region must be completely foreground.
+            Other values are in between, for instance if 0.5, the region must be at least 50% foreground.
+        output_tile_size: tuple[int, int], optional
+            If this value is set, this value will be used as the tile size of the output tiles. If this value
+            is different from the underlying grid, this tile will be extracted around the center of the region.
         rois :
             Regions of interest to restrict the grids to. Coordinates should be given at level 0.
         annotations :
@@ -385,6 +433,8 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
             Transform to be applied to the sample.
         backend :
             Backend to use to read the whole slide image.
+        **kwargs :
+            Gets passed to the SlideImage constructor.
 
         Examples
         --------
@@ -396,7 +446,7 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
         Calling this dataset with an index will return a tile extracted straight from the WSI. This means tiling as
         pre-processing step is not required.
         """
-        with SlideImage.from_file_path(path, backend=backend) as slide_image:
+        with SlideImage.from_file_path(path, backend=backend, **kwargs) as slide_image:
             slide_level_size = slide_image.get_scaled_size(slide_image.get_scaling(mpp))
             slide_mpp = slide_image.mpp
             _rois = parse_rois(rois, slide_level_size, scaling=slide_mpp / mpp if mpp else 1.0)
@@ -420,10 +470,12 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
             crop=crop,
             mask=mask,
             mask_threshold=mask_threshold,
+            output_tile_size=output_tile_size,
             annotations=annotations,
             labels=labels,
             transform=transform,
             backend=backend,
+            **kwargs,
         )
 
     def __getitem__(self, index):
@@ -442,7 +494,7 @@ class TiledROIsSlideImageDataset(SlideImageDatasetBase[RegionFromSlideDatasetSam
         return region_data
 
 
-def parse_rois(rois, image_size, scaling: float) -> Tuple[Tuple[Tuple[int, int], Tuple[int, int]], ...]:
+def parse_rois(rois, image_size, scaling: float) -> tuple[tuple[tuple[int, int], tuple[int, int]], ...]:
     if rois is None:
         return (((0, 0), image_size),)
     else:
